==> nodejs_interop-0.8.0.tar.gz: lib/src/io/file/random_access_file.dart <==
part of nodejs_interop.io;

/// Provides random access to the data in a file.
class RandomAccessFile implements f.RandomAccessFile {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// The file descriptor of the underlying file.
  final int _fd;

  /// The current byte position in the file.
  int _position = 0;

  /// Creates a new random access file.
  RandomAccessFile._internal(this._fd, this.path);

  /// The path of the underlying file.
  @override
  final String path;

  /// Closes the file.
  /// Returns a [Future] that completes with this [RandomAccessFile] when it has been closed.
  @override
  Future<RandomAccessFile> close() async {
    try {
      await fn1cb0ToFuture(_fs.close, _fd);
      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously closes the file.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void closeSync() {
    try { _fs.closeSync(_fd); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Flushes the contents of the file to disk.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the flush operation completes.
  @override
  Future<RandomAccessFile> flush() async {
    try {
      await fn1cb0ToFuture(_fs.fsync, _fd);
      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously flushes the contents of the file to disk.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void flushSync() {
    try { _fs.fsyncSync(_fd); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Gets the length of the file.
  /// Returns a [Future] that completes with the length in bytes.
  @override
  Future<int> length() async {
    var stats = await FileStat.stat(path);
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.size;
  }

  /// Synchronously gets the length of the file.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  int lengthSync() {
    var stats = FileStat.statSync(path);
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.size;
  }

  /// Locks the file or part of the file.
  @override
  Future<RandomAccessFile> lock([f.FileLock mode = f.FileLock.EXCLUSIVE, int start = 0, int end = -1]) =>
    throw new UnsupportedError('Not supported by Node.js');

  /// Synchronously locks the file or part of the file.
  @override
  RandomAccessFile lockSync([f.FileLock mode = f.FileLock.EXCLUSIVE, int start = 0, int end = -1]) =>
    throw new UnsupportedError('Not supported by Node.js');

  /// Gets the current byte position in the file. Returns a [Future] that completes with the position.
  @override
  Future<int> position() => new Future<int>.value(_position);

  /// Synchronously gets the current byte position in the file.
  @override
  int positionSync() => _position;

  /// Reads a maximum of [bytes] bytes from a file. Returns a [Future] that completes with the result in a list of bytes.
  @override
  Future<List<int>> read(int bytes) async {
    try {
      var buffer = new Uint8List(0); // TODO ? new Uint8List(bytes)
      var result = await fn5cb2ToFuture<int, Uint8List>(_fs.read, _fd, buffer, 0, bytes, _position);
      _position += result.item1;
      return result.item2;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Reads a byte from the file. Returns a [Future] that completes with the byte, or with `-1` if end-of-file has been reached.
  @override
  Future<int> readByte() async {
    var bytes = await read(1);
    return bytes.isNotEmpty ? bytes.first : -1;
  }

  /// Synchronously reads a byte from the file. If end-of-file has been reached, `-1` is returned.
  @override
  int readByteSync() {
    var bytes = readSync(1);
    return bytes.isNotEmpty ? bytes.first : -1;
  }

  /// Reads into an existing [List] from the file.
  ///
  /// If [start] is present, the bytes will be filled into [buffer] from at index [start], otherwise index `0`.
  /// If [end] is present, the `end - start` bytes will be read into [buffer], otherwise up to `buffer.length`.
  /// If `end == start` nothing happens.
  ///
  /// Returns a [Future] that completes with the number of bytes read.
  @override
  Future<int> readInto(List<int> buffer, [int start = 0, int end]) async {
    end ??= buffer.length;

    var bytes = await read(end - start);
    buffer.setRange(start, end, bytes);
    return bytes.length;
  }

  /// Synchronously reads into an existing [List] from the file.
  ///
  /// If [start] is present, the bytes will be filled into [buffer] from at index [start], otherwise index `0`.
  /// If [end] is present, the `end - start` bytes will be read into [buffer], otherwise up to `buffer.length`.
  /// If `end == start` nothing happens.
  ///
  /// Returns the number of bytes read. Throws a [FileSystemException] if the operation fails.
  @override
  int readIntoSync(List<int> buffer, [int start = 0, int end]) {
    end ??= buffer.length;

    var bytes = readSync(end - start);
    buffer.setRange(start, end, bytes);
    return bytes.length;
  }

  /// Synchronously reads a maximum of [bytes] bytes from a file and returns the result in a list of bytes.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  List<int> readSync(int bytes) {
    try {
      var buffer = new Uint8List(0); // TODO ? new Uint8List(bytes)
      _position += _fs.readSync(_fd, buffer, 0, bytes, _position);
      return buffer;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Sets the byte position in the file.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the position has been set.
  @override
  Future<RandomAccessFile> setPosition(int position) {
    _position = position;
    return new Future<RandomAccessFile>.value(this);
  }

  /// Synchronously sets the byte position in the file.
  @override
  void setPositionSync(int position) => _position = position;

  /// Returns a string representation of this object.
  @override
  String toString() => 'RandomAccessFile("$path")';

  /// Truncates (or extends) the file to [length] bytes.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the truncation has been performed.
  @override
  Future<RandomAccessFile> truncate(int length) async {
    try {
      await fn2cb0ToFuture(_fs.ftruncate, _fd, length);
      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously truncates (or extends) the file to [length] bytes.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void truncateSync(int length) {
    try { _fs.ftruncateSync(_fd, length); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Unlocks the file or part of the file.
  @override
  Future<RandomAccessFile> unlock([int start = 0, int end = -1]) => throw new UnsupportedError('Not supported by Node.js');

  /// Synchronously unlocks the file or part of the file.
  @override
  void unlockSync([int start = 0, int end = -1]) => throw new UnsupportedError('Not supported by Node.js');

  /// Writes a single byte to the file.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the write completes.
  @override
  Future<RandomAccessFile> writeByte(int value) => writeFrom([value]);

  /// Synchronously writes a single byte to the file.
  /// Returns the number of bytes successfully written. Throws a [FileSystemException] if the operation fails.
  @override
  int writeByteSync(int value) {
    var previousPosition = _position;
    writeFromSync([value]);
    return _position - previousPosition;
  }

  /// Writes from a [List] to the file.
  /// It will read the [buffer] from index [start] to index [end].
  /// If [start] is omitted, it'll start from index `0`. If [end] is omitted, it will write to the end of [buffer].
  ///
  /// Returns a [Future] that completes with this [RandomAccessFile] when the write completes.
  @override
  Future<RandomAccessFile> writeFrom(List<int> buffer, [int start = 0, int end]) async {
    end ??= buffer.length;

    try {
      var data = buffer is Uint8List ? buffer : new Uint8List.fromList(buffer);
      var result = await fn5cb2ToFuture<int, Uint8List>(_fs.write, _fd, data, start, end - start, _position);
      _position += result.item1;
      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously writes from a [List] to the file.
  /// It will read the [buffer] from index [start] to index [end].
  /// If [start] is omitted, it'll start from index `0`. If [end] is omitted, it will write to the end of [buffer].
  ///
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void writeFromSync(List<int> buffer, [int start = 0, int end]) {
    end ??= buffer.length;
    try { _position += _fs.writeSync(_fd, buffer is Uint8List ? buffer : new Uint8List.fromList(buffer), start, end - start, _position); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Writes a single [string] to the file using the given [encoding].
  /// Returns a [Future] that completes with this [RandomAccessFile] when the write completes.
  @override
  Future<RandomAccessFile> writeString(String string, {Encoding encoding: UTF8}) => writeFrom(encoding.encode(string));

  /// Synchronously writes a single [string] to the file using the given [encoding].
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void writeStringSync(String string, {Encoding encoding: UTF8}) => writeFromSync(encoding.encode(string));
}

==> nodejs_interop-0.9.0.tar.gz: lib/src/io/file/random_access_file.dart <==
part of nodejs_interop.io;

/// Provides random access to the data in a file.
class RandomAccessFile implements f.RandomAccessFile {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// The file descriptor of the underlying file.
  final int _fd;

  /// The current byte position in the file.
  int _position = 0;

  /// Creates a new random access file.
  RandomAccessFile._internal(this._fd, this.path);

  /// The path of the underlying file.
  @override
  final String path;

  /// Closes the file.
  /// Returns a [Future] that completes with this [RandomAccessFile] when it has been closed.
  @override
  Future<RandomAccessFile> close() {
    var completer = new Completer<RandomAccessFile>();
    _fs.close(_fd, allowInterop(([error]) {
      if (error == null) completer.complete(this);
      else completer.completeError(new f.FileSystemException(error.message, path));
    }));

    return completer.future;
  }

  /// Synchronously closes the file.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void closeSync() {
    try { _fs.closeSync(_fd); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Flushes the contents of the file to disk.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the flush operation completes.
  @override
  Future<RandomAccessFile> flush() {
    var completer = new Completer<RandomAccessFile>();
    _fs.fsync(_fd, allowInterop(([error]) {
      if (error == null) completer.complete(this);
      else completer.completeError(new f.FileSystemException(error.message, path));
    }));

    return completer.future;
  }

  /// Synchronously flushes the contents of the file to disk.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void flushSync() {
    try { _fs.fsyncSync(_fd); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Gets the length of the file.
  /// Returns a [Future] that completes with the length in bytes.
  @override
  Future<int> length() async {
    var stats = await FileStat.stat(path);
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.size;
  }

  /// Synchronously gets the length of the file.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  int lengthSync() {
    var stats = FileStat.statSync(path);
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.size;
  }

  /// Locks the file or part of the file.
  @override
  Future<RandomAccessFile> lock([f.FileLock mode = f.FileLock.EXCLUSIVE, int start = 0, int end = -1]) =>
    throw new UnsupportedError('Not supported by Node.js');

  /// Synchronously locks the file or part of the file.
  @override
  RandomAccessFile lockSync([f.FileLock mode = f.FileLock.EXCLUSIVE, int start = 0, int end = -1]) =>
    throw new UnsupportedError('Not supported by Node.js');

  /// Gets the current byte position in the file. Returns a [Future] that completes with the position.
  @override
  Future<int> position() => new Future<int>.value(_position);

  /// Synchronously gets the current byte position in the file.
  @override
  int positionSync() => _position;

  /// Reads a maximum of [bytes] bytes from a file. Returns a [Future] that completes with the result in a list of bytes.
  @override
  Future<List<int>> read(int bytes) {
    var buffer = new Uint8List(0); // TODO ? new Uint8List(bytes)
    var completer = new Completer<Uint8List>();

    _fs.read(_fd, buffer, 0, bytes, _position, allowInterop((error, [bytesWritten, buffer]) {
      if (error != null) completer.completeError(new f.FileSystemException(error.message, path));
      else {
        _position += bytesWritten;
        completer.complete(buffer);
      }
    }));

    return completer.future;
  }

  /// Reads a byte from the file. Returns a [Future] that completes with the byte, or with `-1` if end-of-file has been reached.
  @override
  Future<int> readByte() async {
    var bytes = await read(1);
    return bytes.isNotEmpty ? bytes.first : -1;
  }

  /// Synchronously reads a byte from the file. If end-of-file has been reached, `-1` is returned.
  @override
  int readByteSync() {
    var bytes = readSync(1);
    return bytes.isNotEmpty ? bytes.first : -1;
  }

  /// Reads into an existing [List] from the file.
  ///
  /// If [start] is present, the bytes will be filled into [buffer] from at index [start], otherwise index `0`.
  /// If [end] is present, the `end - start` bytes will be read into [buffer], otherwise up to `buffer.length`.
  /// If `end == start` nothing happens.
  ///
  /// Returns a [Future] that completes with the number of bytes read.
  @override
  Future<int> readInto(List<int> buffer, [int start = 0, int end]) async {
    end ??= buffer.length;

    var bytes = await read(end - start);
    buffer.setRange(start, end, bytes);
    return bytes.length;
  }

  /// Synchronously reads into an existing [List] from the file.
  ///
  /// If [start] is present, the bytes will be filled into [buffer] from at index [start], otherwise index `0`.
  /// If [end] is present, the `end - start` bytes will be read into [buffer], otherwise up to `buffer.length`.
  /// If `end == start` nothing happens.
  ///
  /// Returns the number of bytes read. Throws a [FileSystemException] if the operation fails.
  @override
  int readIntoSync(List<int> buffer, [int start = 0, int end]) {
    end ??= buffer.length;

    var bytes = readSync(end - start);
    buffer.setRange(start, end, bytes);
    return bytes.length;
  }

  /// Synchronously reads a maximum of [bytes] bytes from a file and returns the result in a list of bytes.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  List<int> readSync(int bytes) {
    try {
      var buffer = new Uint8List(0); // TODO ? new Uint8List(bytes)
      _position += _fs.readSync(_fd, buffer, 0, bytes, _position);
      return buffer;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Sets the byte position in the file.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the position has been set.
  @override
  Future<RandomAccessFile> setPosition(int position) {
    _position = position;
    return new Future<RandomAccessFile>.value(this);
  }

  /// Synchronously sets the byte position in the file.
  @override
  void setPositionSync(int position) => _position = position;

  /// Returns a string representation of this object.
  @override
  String toString() => 'RandomAccessFile("$path")';

  /// Truncates (or extends) the file to [length] bytes.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the truncation has been performed.
  @override
  Future<RandomAccessFile> truncate(int length) {
    var completer = new Completer<RandomAccessFile>();
    _fs.ftruncate(_fd, length, allowInterop(([error]) {
      if (error == null) completer.complete(this);
      else completer.completeError(new f.FileSystemException(error.message, path));
    }));

    return completer.future;
  }

  /// Synchronously truncates (or extends) the file to [length] bytes.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void truncateSync(int length) {
    try { _fs.ftruncateSync(_fd, length); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Unlocks the file or part of the file.
  @override
  Future<RandomAccessFile> unlock([int start = 0, int end = -1]) => throw new UnsupportedError('Not supported by Node.js');

  /// Synchronously unlocks the file or part of the file.
  @override
  void unlockSync([int start = 0, int end = -1]) => throw new UnsupportedError('Not supported by Node.js');

  /// Writes a single byte to the file.
  /// Returns a [Future] that completes with this [RandomAccessFile] when the write completes.
  @override
  Future<RandomAccessFile> writeByte(int value) => writeFrom([value]);

  /// Synchronously writes a single byte to the file.
  /// Returns the number of bytes successfully written. Throws a [FileSystemException] if the operation fails.
  @override
  int writeByteSync(int value) {
    var previousPosition = _position;
    writeFromSync([value]);
    return _position - previousPosition;
  }

  /// Writes from a [List] to the file.
  /// It will read the [buffer] from index [start] to index [end].
  /// If [start] is omitted, it'll start from index `0`. If [end] is omitted, it will write to the end of [buffer].
  ///
  /// Returns a [Future] that completes with this [RandomAccessFile] when the write completes.
  @override
  Future<RandomAccessFile> writeFrom(List<int> buffer, [int start = 0, int end]) {
    end ??= buffer.length;
    var completer = new Completer<RandomAccessFile>();
    var data = buffer is Uint8List ? buffer : new Uint8List.fromList(buffer);

    _fs.write(_fd, data, start, end - start, _position, allowInterop((error, [bytesWritten, buffer]) {
      if (error != null) completer.completeError(new f.FileSystemException(error.message, path));
      else {
        _position += bytesWritten;
        completer.complete(this);
      }
    }));

    return completer.future;
  }

  /// Synchronously writes from a [List] to the file.
  /// It will read the [buffer] from index [start] to index [end].
  /// If [start] is omitted, it'll start from index `0`. If [end] is omitted, it will write to the end of [buffer].
  ///
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void writeFromSync(List<int> buffer, [int start = 0, int end]) {
    end ??= buffer.length;
    try { _position += _fs.writeSync(_fd, buffer is Uint8List ? buffer : new Uint8List.fromList(buffer), start, end - start, _position); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Writes a single [string] to the file using the given [encoding].
  /// Returns a [Future] that completes with this [RandomAccessFile] when the write completes.
  @override
  Future<RandomAccessFile> writeString(String string, {Encoding encoding = utf8}) => writeFrom(encoding.encode(string));

  /// Synchronously writes a single [string] to the file using the given [encoding].
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void writeStringSync(String string, {Encoding encoding = utf8}) => writeFromSync(encoding.encode(string));
}
